/* ntdisp - NtD In System Programming.
 *
 * Copyright (C) 2012  Nicola Fontana <ntd at entidi.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library. If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef __NTD_DEVICE_H__
#define __NTD_DEVICE_H__


G_BEGIN_DECLS

#define NTD_TYPE_DEVICE             (ntd_device_get_type ())
#define NTD_DEVICE(obj)             (G_TYPE_CHECK_INSTANCE_CAST ((obj), NTD_TYPE_DEVICE, NtdDevice))
#define NTD_DEVICE_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST ((klass), NTD_TYPE_DEVICE, NtdDeviceClass))
#define NTD_IS_DEVICE(obj)          (G_TYPE_CHECK_INSTANCE_TYPE ((obj), NTD_TYPE_DEVICE))
#define NTD_IS_DEVICE_CLASS(klass)  (G_TYPE_CHECK_CLASS_TYPE ((klass), NTD_TYPE_DEVICE))
#define NTD_DEVICE_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj), NTD_TYPE_DEVICE, NtdDeviceClass))

typedef struct _NtdDevice       NtdDevice;
typedef struct _NtdDeviceClass  NtdDeviceClass;

struct _NtdDevice {
    /*< private >*/
    GObject             object;
    gpointer            data;
};

struct _NtdDeviceClass {
    /*< private >*/
    GObjectClass   parent_class;

    /* Signals */
    void                  (* open)              (NtdDevice*     device);
    void                  (* close)             (NtdDevice*     device);
    void                  (* flush)             (NtdDevice*     device,
                                                 gboolean       input,
                                                 gboolean       output);
    gsize                 (* raw_read)          (NtdDevice*     device,
                                                 gsize          nbytes,
                                                 gpointer       buffer);
    gsize                 (* raw_write)         (NtdDevice*     device,
                                                 gsize          nbytes,
                                                 gconstpointer  buffer);
    void                  (* error_set)         (NtdDevice*     device);
};


GType           ntd_device_get_type             (void) G_GNUC_CONST;

void            ntd_device_set_error            (NtdDevice*     device,
                                                 NtdSystemError error_code,
                                                 const gchar*   format,
                                                 ...) G_GNUC_PRINTF (3, 4);
void            ntd_device_set_errorv           (NtdDevice*     device,
                                                 NtdSystemError error_code,
                                                 const gchar*   format,
                                                 va_list        args);
gboolean        ntd_device_set_error_from_system(NtdDevice*     device,
                                                 const gchar*   format,
                                                 ...) G_GNUC_PRINTF (2, 3);
GError*         ntd_device_get_error            (NtdDevice*     device);
gboolean        ntd_device_clear_error          (NtdDevice*     device);
gboolean        ntd_device_in_error             (NtdDevice*     device);

void            ntd_device_set_file             (NtdDevice*     device,
                                                 const gchar*   file);
const gchar*    ntd_device_get_file             (NtdDevice*     device);
void            ntd_device_open                 (NtdDevice*     device);
void            ntd_device_close                (NtdDevice*     device);
gboolean        ntd_device_is_opened            (NtdDevice*     device);
void            ntd_device_flush                (NtdDevice*     device,
                                                 gboolean       input,
                                                 gboolean       output);
gpointer        ntd_device_read                 (NtdDevice*     device,
                                                 gsize          nbytes);
gpointer        ntd_device_read_until           (NtdDevice*     device,
                                                 gchar          eol);
gboolean        ntd_device_write                (NtdDevice*     device,
                                                 gsize          nbytes,
                                                 gconstpointer  buffer);
gboolean        ntd_device_printf               (NtdDevice*     device,
                                                 const gchar*   format,
                                                 ...) G_GNUC_PRINTF(2, 3);
gboolean        ntd_device_vprintf              (NtdDevice*     device,
                                                 const gchar*   format,
                                                 va_list        var_args);

G_END_DECLS


#endif /* __NTD_DEVICE_H__ */
