/* ntdisp - NtD In System Programming.
 *
 * Copyright (C) 2012  Nicola Fontana <ntd at entidi.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library. If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef __NTD_ISP_H__
#define __NTD_ISP_H__


G_BEGIN_DECLS

#define NTD_TYPE_ISP             (ntd_isp_get_type())
#define NTD_ISP(obj)             (G_TYPE_CHECK_INSTANCE_CAST((obj), NTD_TYPE_ISP, NtdIsp))
#define NTD_ISP_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST((klass), NTD_TYPE_ISP, NtdIspClass))
#define NTD_IS_ISP(obj)          (G_TYPE_CHECK_INSTANCE_TYPE((obj), NTD_TYPE_ISP))
#define NTD_IS_ISP_CLASS(klass)  (G_TYPE_CHECK_CLASS_TYPE((klass), NTD_TYPE_ISP))
#define NTD_ISP_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS((obj), NTD_TYPE_ISP, NtdIspClass))

typedef struct _NtdIsp        NtdIsp;
typedef struct _NtdIspClass   NtdIspClass;

struct _NtdIsp {
    /*< private >*/
    GObject              parent;
    gpointer             data;
};

struct _NtdIspClass {
    /*< private >*/
    GObjectClass         parent_class;
    /* Signals */
    void                (* status_changed)      (NtdIsp*        isp);
    void                (* progress_notify)     (NtdIsp*        isp,
                                                 gdouble        fraction);
    /* Virtual Table */
    void                (* set_defaults)        (NtdIsp*        isp);
    const gchar*        (* status_to_message)   (gint           status);
    void                (* connect)             (NtdIsp*        isp);
    void                (* disconnect)          (NtdIsp*        isp);
    gboolean            (* probe)               (NtdIsp*        isp);
    gboolean            (* wipe)                (NtdIsp*        isp,
                                                 const gchar*   address,
                                                 gsize          nbytes);
    gboolean            (* download)            (NtdIsp*        isp,
                                                 const gchar*   address,
                                                 gconstpointer  buffer,
                                                 gsize          nbytes);
    gboolean            (* dump)                (NtdIsp*        isp,
                                                 const gchar*   address,
                                                 gpointer*      buffer,
                                                 gsize*         nbytes);
    gboolean            (* run)                 (NtdIsp*        isp,
                                                 const gchar*   address);
};


GType           ntd_isp_get_type                (void) G_GNUC_CONST;

void            ntd_isp_set_device              (NtdIsp*        isp,
                                                 const gchar*   file);
void            ntd_isp_set_serial              (NtdIsp*        isp,
                                                 NtdSerial*     serial);
NtdSerial*      ntd_isp_get_serial              (NtdIsp*        isp);
void            ntd_isp_set_status              (NtdIsp*        isp,
                                                 gint           status);
void            ntd_isp_clear_status            (NtdIsp*        isp);
gint            ntd_isp_get_status              (NtdIsp*        isp);
const gchar*    ntd_isp_get_message             (NtdIsp*        isp);
void            ntd_isp_status_changed          (NtdIsp*        isp);
void            ntd_isp_progress_notify         (NtdIsp*        isp,
                                                 gdouble        fraction);

void            ntd_isp_connect                 (NtdIsp*        isp);
void            ntd_isp_disconnect              (NtdIsp*        isp);
void            ntd_isp_set_connected           (NtdIsp*        isp,
                                                 gboolean       connected);
gboolean        ntd_isp_is_connected            (NtdIsp*        isp);
gboolean        ntd_isp_probe                   (NtdIsp*        isp);
gboolean        ntd_isp_wipe                    (NtdIsp*        isp,
                                                 const gchar*   address,
                                                 gsize          nbytes);
gboolean        ntd_isp_download                (NtdIsp*        isp,
                                                 const gchar*   address,
                                                 gconstpointer  buffer,
                                                 gsize          nbytes);
gboolean        ntd_isp_download_file           (NtdIsp*        isp,
                                                 const gchar*   address,
                                                 const gchar*   file,
                                                 GError**       error);
gboolean        ntd_isp_dump                    (NtdIsp*        isp,
                                                 const gchar*   address,
                                                 gpointer*      buffer,
                                                 gsize*         nbytes);
gboolean        ntd_isp_dump_file               (NtdIsp*        isp,
                                                 const gchar*   address,
                                                 gsize          nbytes,
                                                 const gchar*   file,
                                                 GError**       error);
gboolean        ntd_isp_verify                  (NtdIsp*        isp,
                                                 const gchar*   address,
                                                 gconstpointer  buffer,
                                                 gsize          nbytes);
gboolean        ntd_isp_verify_file             (NtdIsp*        isp,
                                                 const gchar*   address,
                                                 const gchar*   file,
                                                 GError**       error);
gboolean        ntd_isp_run                     (NtdIsp*        isp,
                                                 const gchar*   address);

G_END_DECLS


#endif /* __NTD_ISP_H__ */
