/* ntdisp - NtD In System Programming.
 *
 * Copyright (C) 2012  Nicola Fontana <ntd at entidi.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library. If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef __NTD_LPC_H__
#define __NTD_LPC_H__


G_BEGIN_DECLS

#define NTD_TYPE_LPC             (ntd_lpc_get_type())
#define NTD_LPC(obj)             (G_TYPE_CHECK_INSTANCE_CAST((obj), NTD_TYPE_LPC, NtdLpc))
#define NTD_LPC_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST((klass), NTD_TYPE_LPC, NtdLpcClass))
#define NTD_IS_LPC(obj)          (G_TYPE_CHECK_INSTANCE_TYPE((obj), NTD_TYPE_LPC))
#define NTD_IS_LPC_CLASS(klass)  (G_TYPE_CHECK_CLASS_TYPE((klass), NTD_TYPE_LPC))
#define NTD_LPC_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS((obj), NTD_TYPE_LPC, NtdLpcClass))

typedef struct _NtdLpc        NtdLpc;
typedef struct _NtdLpcClass   NtdLpcClass;

struct _NtdLpc {
    /*< private >*/
    NtdIsp             parent;
    gpointer             data;
};

struct _NtdLpcClass {
    /*< private >*/
    NtdIspClass parent_class;
};


GType           ntd_lpc_get_type            (void) G_GNUC_CONST;
NtdLpc*         ntd_lpc_new                 (void);
NtdLpc*         ntd_lpc_new_with_device     (const gchar*       device);

gboolean        ntd_lpc_config_by_name      (NtdLpc*            lpc,
                                             const gchar*       name);
gboolean        ntd_lpc_config_by_device_id (NtdLpc*            lpc,
                                             guint32            device_id);
void            ntd_lpc_set_name            (NtdLpc*            lpc,
                                             const gchar*       name);
const gchar*    ntd_lpc_get_name            (NtdLpc*            lpc);
void            ntd_lpc_set_buffer_address  (NtdLpc*            lpc,
                                             gulong             address);
gulong          ntd_lpc_get_buffer_address  (NtdLpc*            lpc);
void            ntd_lpc_set_buffer_size     (NtdLpc*            lpc,
                                             guint              size);
guint           ntd_lpc_get_buffer_size     (NtdLpc*            lpc);
void            ntd_lpc_set_chunk_size      (NtdLpc*            lpc,
                                             guint              chunk_size);
guint           ntd_lpc_get_chunk_size      (NtdLpc*            lpc);
void            ntd_lpc_set_frequency       (NtdLpc*            lpc,
                                             guint              frequency);
guint           ntd_lpc_get_frequency       (NtdLpc*            lpc);

void            ntd_lpc_set_flash_table     (NtdLpc*            lpc,
                                             const gchar*       flash_table);
const gchar*    ntd_lpc_get_flash_table     (NtdLpc*            lpc);
gsize           ntd_lpc_get_flash_size      (NtdLpc*            lpc);
gboolean        ntd_lpc_command             (NtdLpc*            lpc,
                                             const gchar*       format,
                                             ...) G_GNUC_PRINTF(2, 3);
gboolean        ntd_lpc_command_valist      (NtdLpc*            lpc,
                                             const gchar*       format,
                                             va_list            var_args);
gboolean        ntd_lpc_encode              (NtdLpc*            lpc,
                                             gconstpointer      data,
                                             guint              nbytes);
gpointer        ntd_lpc_decode              (NtdLpc*            lpc,
                                             guint              nbytes);
gchar *         ntd_lpc_read_line           (NtdLpc*            lpc);
gboolean        ntd_lpc_answered_OK         (NtdLpc*            lpc);
gboolean        ntd_lpc_answered_0          (NtdLpc*            lpc);
gboolean        ntd_lpc_answered_data       (NtdLpc*            lpc,
                                             gchar**            p_data,
                                             gulong*            p_checksum);
gint            ntd_lpc_map_address         (NtdLpc*            lpc,
                                             gulong             address);
guint           ntd_lpc_ram_to_flash        (NtdLpc*            lpc,
                                             gulong             ram,
                                             gulong             flash,
                                             guint              size);

G_END_DECLS


#endif /* __NTD_LPC_H__ */
