/* ntdisp - NtD In System Programming.
 *
 * Copyright (C) 2012  Nicola Fontana <ntd at entidi.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library. If not, see <http://www.gnu.org/licenses/>.
 */


#include <glib-object.h>

#include "../config.h"
#include "ntdisp-utils.h"
#include "ntd-device.h"
#include "ntd-null.h"


static void     ntd_null_class_init(NtdNullClass   *klass);

static GObjectClass *parent_class = NULL;


GType
ntd_null_get_type(void)
{
    static GType null_type = 0;

    if G_UNLIKELY(null_type == 0) {
        static const GTypeInfo null_info = {
            sizeof(NtdNullClass),
            NULL,                   /* base_init */
            NULL,                   /* base_finalize */
            (GClassInitFunc)        ntd_null_class_init,
            NULL,                   /* class_finalize */
            NULL,                   /* class_data */
            sizeof(NtdNull),
            0,                      /* n_preallocs */
            NULL,                   /* instance_init */
        };

        null_type = g_type_register_static(NTD_TYPE_DEVICE, "NtdNull",
                                           &null_info, 0);
    }

    return null_type;
}

static void
ntd_null_class_init(NtdNullClass *klass)
{
    NtdDeviceClass *device_class;

    parent_class = g_type_class_peek_parent(klass);

    device_class = (NtdDeviceClass *) klass;

    device_class->open = NULL;
    device_class->close = NULL;
    device_class->raw_read = NULL;
    device_class->raw_write = NULL;
}


/**
 * ntd_null_new:
 *
 * Creates a new #NtdNull device.
 *
 * Returns: the newly created object.
 */
NtdDevice *
ntd_null_new(void)
{
    return (NtdDevice *) g_object_new(NTD_TYPE_NULL, NULL);
}

