/* ntdisp - NtD In System Programming.
 *
 * Copyright (C) 2012  Nicola Fontana <ntd at entidi.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library. If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef __NTD_SERIAL_H__
#define __NTD_SERIAL_H__


G_BEGIN_DECLS

#define NTD_TYPE_SERIAL             (ntd_serial_get_type ())
#define NTD_SERIAL(obj)             (G_TYPE_CHECK_INSTANCE_CAST ((obj), NTD_TYPE_SERIAL, NtdSerial))
#define NTD_SERIAL_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST ((klass), NTD_TYPE_SERIAL, NtdSerialClass))
#define NTD_IS_SERIAL(obj)          (G_TYPE_CHECK_INSTANCE_TYPE ((obj), NTD_TYPE_SERIAL))
#define NTD_IS_SERIAL_CLASS(klass)  (G_TYPE_CHECK_CLASS_TYPE ((klass), NTD_TYPE_SERIAL))
#define NTD_SERIAL_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj), NTD_TYPE_SERIAL, NtdSerialClass))


typedef struct _NtdSerial           NtdSerial;
typedef struct _NtdSerialClass      NtdSerialClass;


struct _NtdSerial {
    /*< private >*/
    NtdDevice              device;
    gpointer               data;
};

struct _NtdSerialClass {
    /*< private >*/
    NtdDeviceClass         parent_class;
};


GType           ntd_serial_get_type             (void) G_GNUC_CONST;
NtdSerial*      ntd_serial_new                  (const gchar*   file);

void            ntd_serial_set                  (NtdSerial*     serial,
                                                 NtdBaud        baud,
                                                 NtdParity      parity,
                                                 NtdDataBits    data_bits,
                                                 NtdStopBits    stop_bits);
void            ntd_serial_set_exclusive        (NtdSerial*     serial,
                                                 gboolean       exclusive);
gboolean        ntd_serial_get_exclusive        (NtdSerial*     serial);
void            ntd_serial_set_baud             (NtdSerial*     serial,
                                                 NtdBaud        baud);
NtdBaud         ntd_serial_get_baud             (NtdSerial*     serial);
void            ntd_serial_set_parity           (NtdSerial*     serial,
                                                 NtdParity      parity);
NtdParity       ntd_serial_get_parity           (NtdSerial*     serial);
void            ntd_serial_set_data_bits        (NtdSerial*     serial,
                                                 NtdDataBits    data_bits);
NtdDataBits     ntd_serial_get_data_bits        (NtdSerial*     serial);
void            ntd_serial_set_stop_bits        (NtdSerial*     serial,
                                                 NtdStopBits    stop_bits);
NtdStopBits     ntd_serial_get_stop_bits        (NtdSerial*     serial);
void            ntd_serial_set_software_flow    (NtdSerial*     serial,
                                                 NtdSoftwareFlow software_flow);
NtdSoftwareFlow ntd_serial_get_software_flow    (NtdSerial*     serial);
void            ntd_serial_set_hardware_flow    (NtdSerial*     serial,
                                                 NtdHardwareFlow hardware_flow);
NtdHardwareFlow ntd_serial_get_hardware_flow    (NtdSerial*     serial);
void            ntd_serial_set_timeout          (NtdSerial*     serial,
                                                 gdouble        timeout);
gdouble         ntd_serial_get_timeout          (NtdSerial*     serial);

G_END_DECLS


#endif /* __NTD_SERIAL_H__ */
