/* ntdisp - NtD In System Programming.
 *
 * Copyright (C) 2012  Nicola Fontana <ntd at entidi.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library. If not, see <http://www.gnu.org/licenses/>.
 */


/**
 * SECTION:ntdisp-enums
 * @Section_Id:enumerations
 * @title: Standard enumerations
 * @short_description: Public enumerated types used throughout ntdisp
 *
 * All the enumerations are available in GType notation, and so are
 * the standard type macros.
 *
 * Since: 1.0
 **/


/**
 * NtdBaud:
 * @NTD_BAUD_UNKNOWN: 0 baud
 * @NTD_BAUD_50:      50 baud
 * @NTD_BAUD_75:      75 baud
 * @NTD_BAUD_110:     110 baud
 * @NTD_BAUD_134:     134 baud
 * @NTD_BAUD_200:     200 baud
 * @NTD_BAUD_300:     300 baud
 * @NTD_BAUD_600:     600 baud
 * @NTD_BAUD_1200:    1200 baud
 * @NTD_BAUD_1800:    1800 baud
 * @NTD_BAUD_2400:    2400 baud
 * @NTD_BAUD_4800:    4800 baud
 * @NTD_BAUD_9600:    9600 baud
 * @NTD_BAUD_19200:   19200 baud
 * @NTD_BAUD_38400:   38400 baud
 * @NTD_BAUD_57600:   57600 baud
 * @NTD_BAUD_115200:  115200 baud
 * @NTD_BAUD_230400:  230400 baud
 * @NTD_BAUD_460800:  460800 baud
 *
 * The number of symbols per second transferred, usually referred as baud
 * rate.
 *
 * Since: 1.0
 **/

/**
 * NtdDataBits:
 * @NTD_DATA_BITS_UNKNOWN: number of data bits is unknown
 * @NTD_DATA_BITS_5:       5 data bits
 * @NTD_DATA_BITS_6:       6 data bits
 * @NTD_DATA_BITS_7:       7 data bits
 * @NTD_DATA_BITS_8:       8 data bits
 *
 * The number of data bits in each symbol. Pretty much all modern devices
 * use 8 data bits with no parity.
 *
 * Since: 1.0
 **/

/**
 * NtdParity:
 * @NTD_PARITY_UNKNOWN: parity check undefined
 * @NTD_PARITY_NONE:    no parity bit present
 * @NTD_PARITY_ODD:     parity bit checked for odd parity
 * @NTD_PARITY_EVEN:    parity bit checked for even parity
 * @NTD_PARITY_UNCHECK: parity bit ignored
 *
 * The parity scheme to be used. Pretty much all modern devices
 * use 8 data bits with no parity.
 *
 * Since: 1.0
 **/

/**
 * NtdStopBits:
 * @NTD_STOP_BITS_UNKNOWN: number of stop bits is unknown
 * @NTD_STOP_BITS_1:       1 stop bit
 * @NTD_STOP_BITS_1_5:     1.5 stop bits
 * @NTD_STOP_BITS_2:       2 stop bits
 *
 * The length, in bits, of the stop signal. This is usually set to 1.
 *
 * Since: 1.0
 **/

/**
 * NtdHardwareFlow:
 * @NTD_HARDWARE_FLOW_NONE:    no hardware flow control
 * @NTD_HARDWARE_FLOW_RTS:     enable emission of "request to send" signal
 * @NTD_HARDWARE_FLOW_CTS:     enable monitor of "clear to send" signal
 * @NTD_HARDWARE_FLOW_RTS_CTS: enable either RTS and CTS signal handling
 *
 * Type of hardware flow control to use in the communication.
 *
 * Since: 1.0
 **/

/**
 * NtdSoftwareFlow:
 * @NTD_SOFTWARE_FLOW_NONE:     no software flow control
 * @NTD_SOFTWARE_FLOW_XON:      enable emission of XON when the input
 *                              buffer is able to receive
 * @NTD_SOFTWARE_FLOW_XOFF:     enable emission of XOFF when the input
 *                              buffer is becoming full
 * @NTD_SOFTWARE_FLOW_XON_XOFF: enable either XON and XOFF handling
 *
 * Type of software flow control to use in the communication.
 *
 * Since: 1.0
 **/

/**
 * NtdLpcModel:
 * @NTD_LPC_MODEL_UNKNOWN: LPC model unknown
 * @NTD_LPC_MODEL_PROBE:   try to send a probe request and get the
 *                         model from the response
 * @NTD_LPC_MODEL_2XXX:    LPC2000 family
 * @NTD_LPC_MODEL_17XX:    LPC1700 family
 * @NTD_LPC_MODEL_13XX:    LPC1300 family
 * @NTD_LPC_MODEL_11XX:    LPC1100 family
 *
 * The model of NXP-LPC based device used as target.
 *
 * Since: 1.0
 **/

/**
 * NtdStmCommand:
 * @NTD_STM_COMMAND_GET:    gets the version and the allowed commands
 *                          supported by the current version of the bootloader
 * @NTD_STM_COMMAND_GET_VERSION: gets the bootloader version and the Read
 *                          Protection status of the Flash memory
 * @NTD_STM_COMMAND_GET_ID: gets the chip ID
 * @NTD_STM_COMMAND_READ:   reads up to 256 bytes of memory starting from an
 *                          address specified by the application
 * @NTD_STM_COMMAND_GO:     jumps to user application code located in the
 *                          internal Flash memory or in SRAM
 * @NTD_STM_COMMAND_WRITE:  writes up to 256 bytes to the RAM or Flash memory
 *                          starting from an address specified by the
 *                          application
 * @NTD_STM_COMMAND_ERASE:  erases from one to all the Flash memory pages
 * @NTD_STM_COMMAND_WRITE_PROCTECT: enables the write protection for some
 *                          sectors
 * @NTD_STM_COMMAND_WRITE_UNPROCTECT: disables the write protection for all
 *                          Flash memory sectors
 * @NTD_STM_COMMAND_READ_PROCTECT: enables the read protection
 * @NTD_STM_COMMAND_READ_UNPROCTECT: disables the read protection
 *
 * Map of commands availables in the #NtdStm implementation of the STM32
 * ISP protocol.
 *
 * Since: 1.0
 **/

/**
 * NtdStmAnswer:
 * @NTD_STM_ANSWER_NONE:    no answer
 * @NTD_STM_ANSWER_ACK:     positive answer
 * @NTD_STM_ANSWER_NAK:     negative answer
 * @NTD_STM_ANSWER_UNKNOWN: malformed or unrecognized answer
 *
 * List of answer recognized by the #NtdStm protocol implementation of
 * the STM32 ISP protocol.
 *
 * Since: 1.0
 **/
