/* ntdisp - NtD In System Programming.
 *
 * Copyright (C) 2012  Nicola Fontana <ntd at entidi.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library. If not, see <http://www.gnu.org/licenses/>.
 */


/**
 * SECTION:ntdisp-utils
 * @Section_Id:utilities
 * @title: Utilities
 * @short_description: Assorted macros and functions
 *
 * Collection of macros and functions that do not fit inside any other topic.
 *
 * Since: 1.0
 **/


#include <glib.h>
#include <glib-object.h>
#include <string.h>
#include <stdio.h>

#include "../config.h"
#include "ntdisp-intl.h"
#include "ntdisp-utils.h"


/**
 * ntd_empty:
 * @string: char pointer
 *
 * Checks wheter @string is empty, that is if @string is %NULL
 * or if its first char is '\0'.
 *
 * Returns: %TRUE is @string is %NULL or if @string[0] is '\0',
 *          %FALSE otherwise.
 **/

/**
 * NtdSystemError:
 * @NTD_SYSTEM_ERROR_NONE: No error present
 * @NTD_SYSTEM_ERROR_FAILED: Generic failure
 * @NTD_SYSTEM_ERROR_PERM: Operation not permitted
 * @NTD_SYSTEM_ERROR_NOENT: No such file or directory
 * @NTD_SYSTEM_ERROR_SRCH: No such process
 * @NTD_SYSTEM_ERROR_INTR: Interrupted system call
 * @NTD_SYSTEM_ERROR_IO: I/O error
 * @NTD_SYSTEM_ERROR_NXIO: No such device or address
 * @NTD_SYSTEM_ERROR_2BIG: Argument list too long
 * @NTD_SYSTEM_ERROR_NOEXEC: Exec format error
 * @NTD_SYSTEM_ERROR_BADF: Bad file number
 * @NTD_SYSTEM_ERROR_CHILD: No child processes
 * @NTD_SYSTEM_ERROR_AGAIN: Try again
 * @NTD_SYSTEM_ERROR_NOMEM: Out of memory
 * @NTD_SYSTEM_ERROR_ACCES: Permission denied
 * @NTD_SYSTEM_ERROR_FAULT: Bad address
 * @NTD_SYSTEM_ERROR_BUSY: Device or resource busy
 * @NTD_SYSTEM_ERROR_EXIST: File exists
 * @NTD_SYSTEM_ERROR_XDEV: Cross-device link
 * @NTD_SYSTEM_ERROR_NODEV: No such device
 * @NTD_SYSTEM_ERROR_NOTDIR: Not a directory
 * @NTD_SYSTEM_ERROR_ISDIR: Is a directory
 * @NTD_SYSTEM_ERROR_INVAL: Invalid argument
 * @NTD_SYSTEM_ERROR_NFILE: File table overflow
 * @NTD_SYSTEM_ERROR_MFILE: Too many open files
 * @NTD_SYSTEM_ERROR_NOTTY: Not a typewriter
 * @NTD_SYSTEM_ERROR_TXTBSY: Text file busy
 * @NTD_SYSTEM_ERROR_FBIG: File too large
 * @NTD_SYSTEM_ERROR_NOSPC: No space left on device
 * @NTD_SYSTEM_ERROR_SPIPE: Illegal seek
 * @NTD_SYSTEM_ERROR_ROFS: Read-only file system
 * @NTD_SYSTEM_ERROR_MLINK: Too many links
 * @NTD_SYSTEM_ERROR_PIPE: Broken pipe
 * @NTD_SYSTEM_ERROR_DOM: Math argument out of domain of func
 * @NTD_SYSTEM_ERROR_RANGE: Math result not representable
 * @NTD_SYSTEM_ERROR_DEADLK: Resource deadlock would occur
 * @NTD_SYSTEM_ERROR_NAMETOOLONG: File name too long
 * @NTD_SYSTEM_ERROR_NOLCK: No record locks available
 * @NTD_SYSTEM_ERROR_NOSYS: Function not implemented
 * @NTD_SYSTEM_ERROR_NOTEMPTY: Directory not empty
 * @NTD_SYSTEM_ERROR_LOOP: Too many symbolic links encountered
 * @NTD_SYSTEM_ERROR_NOMSG: No message of desired type
 * @NTD_SYSTEM_ERROR_IDRM: Identifier removed
 * @NTD_SYSTEM_ERROR_DEADLOCK	EDEADLK
 * @NTD_SYSTEM_ERROR_NOSTR: Device not a stream
 * @NTD_SYSTEM_ERROR_NODATA: No data available
 * @NTD_SYSTEM_ERROR_TIME: Timer expired
 * @NTD_SYSTEM_ERROR_NOSR: Out of streams resources
 * @NTD_SYSTEM_ERROR_PROTO: Protocol error
 * @NTD_SYSTEM_ERROR_BADMSG: Not a data message
 * @NTD_SYSTEM_ERROR_OVERFLOW: Value too large for defined data type
 * @NTD_SYSTEM_ERROR_ILSEQ: Illegal byte sequence
 * @NTD_SYSTEM_ERROR_NOTSOCK: Socket operation on non-socket
 * @NTD_SYSTEM_ERROR_DESTADDRREQ: Destination address required
 * @NTD_SYSTEM_ERROR_MSGSIZE: Message too long
 * @NTD_SYSTEM_ERROR_PROTOTYPE: Protocol wrong type for socket
 * @NTD_SYSTEM_ERROR_NOPROTOOPT: Protocol not available
 * @NTD_SYSTEM_ERROR_PROTONOSUPPORT: Protocol not supported
 * @NTD_SYSTEM_ERROR_OPNOTSUPP: Operation not supported on transport endpoint
 * @NTD_SYSTEM_ERROR_NOTSUP: Operation not supported on transport endpoint
 * @NTD_SYSTEM_ERROR_AFNOSUPPORT: Address family not supported by protocol
 * @NTD_SYSTEM_ERROR_ADDRINUSE: Address already in use
 * @NTD_SYSTEM_ERROR_ADDRNOTAVAIL: Cannot assign requested address
 * @NTD_SYSTEM_ERROR_NETDOWN: Network is down
 * @NTD_SYSTEM_ERROR_NETUNREACH: Network is unreachable
 * @NTD_SYSTEM_ERROR_NETRESET: Network dropped connection because of reset
 * @NTD_SYSTEM_ERROR_NOTCONN: Transport endpoint is not connected
 * @NTD_SYSTEM_ERROR_CONNABORTED: Software caused connection abort
 * @NTD_SYSTEM_ERROR_CONNRESET: Connection reset by peer
 * @NTD_SYSTEM_ERROR_NOBUFS: No buffer space available
 * @NTD_SYSTEM_ERROR_ISCONN: Transport endpoint is already connected
 * @NTD_SYSTEM_ERROR_TIMEDOUT: Connection timed out
 * @NTD_SYSTEM_ERROR_CONNREFUSED: Connection refused
 * @NTD_SYSTEM_ERROR_HOSTUNREACH: No route to host
 * @NTD_SYSTEM_ERROR_ALREADY: Operation already in progress
 * @NTD_SYSTEM_ERROR_INPROGRESS: Operation now in progress
 * @NTD_SYSTEM_ERROR_CANCELED: Operation Canceled
 *
 * Abstraction of the platform dependent error codes. This
 * implementation should be in-line with POSIX.1-2001.
 **/


GQuark
ntd_system_error_quark(void)
{
    static GQuark error_quark = 0;

    if (G_UNLIKELY(! error_quark))
        error_quark = g_quark_from_static_string("NtdSystemError");

    return error_quark;
}


static void
print_handler(const gchar *log_domain, GLogLevelFlags log_level,
              const gchar *message, gpointer user_data)
{
    g_print("%s\n", message);
}


/**
 * ntd_system_set_errorv_with_code:
 * @error: a pointer to the destination #GError*.
 * @error_code: the specific system error code to set.
 * @format: printf-style format string.
 * @args: parameters for format string.
 *
 * Wrapper to ntd_system_set_error_with_code() with #va_list argument.
 **/
void
ntd_system_set_errorv_with_code(GError       **error,
                                NtdSystemError error_code,
                                const gchar   *format,
                                va_list        args)
{
    gchar *message;

    g_return_if_fail(error);

    if (error_code == NTD_SYSTEM_ERROR_NONE) {
        g_clear_error(error);
        return;
    }

    ntd_return_on_pileup(*error);

    /* Retrieving the system error description */
    message = ntd_system_get_error_message(error_code);
    if (! message)
        message = g_strdup_printf(_("Unknown system error `%d'"), error_code);

    /* Prepending the custom message */
    if (format && format[0]) {
        gchar  *new_message;
        gchar  *custom_message;

        custom_message = g_strdup_vprintf(format, args);
        new_message = g_strconcat(custom_message, ": ", message, NULL);

        g_free(custom_message);
        g_free(message);

        message = new_message;
    }

    g_set_error(error, NTD_SYSTEM_ERROR, error_code, "%s", message);

    g_free(message);
}

/**
 * ntd_system_set_error_with_code:
 * @error: a pointer to the destination #GError*.
 * @error_code: the specific system error code to set.
 * @format: printf-style format string.
 * @...: parameters for format string.
 *
 * Sets @error to the specified system error, exactly
 * in the same way the ntd_system_set_error() function does.
 *
 * Of course, in this case the @error_code is not system
 * dependent, so no ntd_system_clear_error() call is performed
 * and nothing is returned.
 *
 * Calling this function with #NTD_SYSTEM_ERROR_NONE as the
 * @error_code it is equivalent to call g_clear_error() on @error.
 */
void
ntd_system_set_error_with_code(GError       **error,
                               NtdSystemError error_code,
                               const gchar   *format,
                               ...)
{
    va_list args;

    va_start(args, format);
    ntd_system_set_errorv_with_code(error, error_code, format, args);
    va_end(args);
}

/**
 * ntd_system_set_errorv:
 * @error: a pointer to the destination #GError*.
 * @format: printf-style format string.
 * @args: parameters for format string.
 *
 * Wrapper to ntd_system_set_error() with #va_list argument.
 *
 * Returns: %TRUE if there is a system error, %FALSE otherwise.
 **/
gboolean
ntd_system_set_errorv(GError     **error,
                      const gchar *format,
                      va_list      args)
{
    NtdSystemError error_code;

    error_code = ntd_system_get_error();
    if (error_code == NTD_SYSTEM_ERROR_NONE)
        return FALSE;

    ntd_system_clear_error();
    ntd_system_set_errorv_with_code(error, error_code, format, args);
    return TRUE;
}

/**
 * ntd_system_set_error:
 * @error: a pointer to the destination #GError*.
 * @format: printf-style format string.
 * @...: parameters for format string.
 *
 * Sets @error to the properly system error, if there is
 * one active. The optional format string is appended
 * to the standard system error message.
 * If there are no system errors, this function does
 * nothing and return %FALSE.
 *
 * After the error is set, a ntd_system_clear_error() call
 * is performed to clear the system error.
 *
 * Returns: %TRUE if there is a system error, %FALSE otherwise.
 */
gboolean
ntd_system_set_error(GError     **error,
                     const gchar *format,
                     ...)
{
    gboolean result;
    va_list  args;

    va_start(args, format);
    result = ntd_system_set_errorv(error, format, args);
    va_end(args);

    return result;
}

/**
 * ntd_set_verbose_mode:
 * @new_status: new verbosity status
 *
 * Enables (if @new_status is %TRUE) or disables (if @new_status
 * is %FALSE) the verbosity mode. In verbose mode, data sent to
 * or received from the device is output to stdout.
 **/
void
ntd_set_verbose_mode(gboolean new_status)
{
    static guint handler = 0;

    if (new_status && handler == 0) {
        handler = g_log_set_handler(G_LOG_DOMAIN, G_LOG_LEVEL_INFO,
                                    print_handler, NULL);
    } else if (! new_status && handler != 0) {
        g_log_remove_handler(G_LOG_DOMAIN, handler);
        handler = 0;
    }
}

/**
 * ntd_escape:
 * @buffer: a buffer
 * @nbytes: size of the buffer
 *
 * Generates a printable representation of the first @nbytes
 * bytes of @buffer. This is done by escaping all non-printable
 * chars.
 *
 * The buffer is split on the '\0' delimiters. Every segment in
 * the resulting list of strings is then escaped with
 * g_strescape(). The final list is then joined with the "\\000"
 * string (octal representation of '\0'): at the end, all non-printable
 * characters (NUL bytes included) are properly escaped.
 *
 * Returns: a newly allocated string: free with g_free()
 *          when no longer needed.
 **/
gchar *
ntd_escape(gconstpointer buffer, gsize nbytes)
{
    const gchar *p1, *p2;
    GSList *slist, *p_slist;;
    gint n;
    gchar **array;
    gchar *tmp_str;

    g_return_val_if_fail(buffer != NULL, NULL);

    if (nbytes == 0)
        return g_strdup("");

    p1 = buffer;
    slist = NULL;
    n = 0;
    for (p1 = buffer; (p2 = memchr(p1, '\0', nbytes)) != NULL; p1 = p2 + 1) {
        slist = g_slist_prepend(slist, g_strescape(p1, NULL));
        nbytes -= p2 - p1 + 1;
        p1 = p2 + 1;
        ++n;
    }

    /* The last string is never NUL terminated. If the buffer coincidentally
     * ends with '\0', a new (empty) string must be added anyway. */
    tmp_str = g_malloc(nbytes + 1);
    memcpy(tmp_str, p1, nbytes);
    tmp_str[nbytes] = '\0';
    slist = g_slist_prepend(slist, g_strescape(tmp_str, NULL));
    g_free(tmp_str);
    ++n;

    /* Create the string array from the GSList */
    array = g_new(gchar*, n + 1);
    array[n--] = NULL;
    for (p_slist = slist; p_slist != NULL; p_slist = p_slist->next)
        array[n--] = p_slist->data;
    g_slist_free(slist);

    tmp_str = g_strjoinv("\\000", array);
    g_strfreev(array);
    return tmp_str;
}

/**
 * ntd_checksum:
 * @data: binary buffer
 * @nbytes: size (in bytes) of the data
 *
 * Sums the first @nbytes bytes of @data and returns the result.
 * No checks against arithmetic overflows are performed.
 *
 * Returns: the computed checksum
 **/
gulong
ntd_checksum(gconstpointer data, gsize nbytes)
{
    gulong result;
    const guchar *ptr;

    g_return_val_if_fail(data != NULL, 0);
    g_return_val_if_fail(nbytes > 0, 0);

    result = 0;
    ptr = data;
    while (nbytes --) {
        result += *ptr;
        ++ ptr;
    }

    return result;
}

/**
 * ntd_find_file:
 * @file: the file to search
 * @...:  a NULL terminated list of paths where to look for
 *        file existence.
 *
 * Searches @file in the provided paths and returns the full
 * path to the first existing match. The check is performed
 * using g_file_test() with the G_FILE_TEST_EXISTS test.
 *
 * Returns: a newly allocated string containing the path
 *          or NULL on errors. Free it with g_free() when
 *          no longer needed.
 **/
gchar *
ntd_find_file(const gchar *file, ...)
{
    va_list var_args;
    gchar *path;
    const gchar *base;

    va_start(var_args, file);

    while ((base = va_arg(var_args, const gchar *)) != NULL) {
        path = g_build_filename(base, file, NULL);
        if (g_file_test(path, G_FILE_TEST_EXISTS))
            return path;
        g_free(path);
    }

    return NULL;
}

/**
 * ntd_string_free:
 * @string: a #GString
 * @p_buffer: destination of the buffer or %NULL
 * @p_len: destionation of the buffer length or %NULL
 *
 * A g_string_free() version that returns either the buffer in
 * whatever @p_buffer and its length in @p_len. Both of them
 * can be %NULL, in which case the data is not returned (but
 * it is freed anyway).
 *
 * Furthermore, the case where @string is %NULL is handled
 * gracefully: @p_buffer and @p_len are set to %NULL and 0
 * respectively (if possible).
 **/
void
ntd_string_free(GString *string, gpointer *p_buffer, gsize *p_len)
{
    gsize len = string != NULL ? string->len : 0;

    if (p_len != NULL)
        *p_len = len;

    if (p_buffer != NULL) {
        if (len > 0) {
            /* Ownership of buffer->str changed:
             * free_segment must be FALSE */
            *p_buffer = string->str;
            g_string_free(string, FALSE);
            string = NULL;
        } else {
            *p_buffer = NULL;
        }
    }

    if (string != NULL)
        g_string_free(string, TRUE);
}

/**
 * ntd_free_and_nullify:
 * @mem: pointer to the pointer to free.
 *
 * Convenient function to free @mem with g_free() and set
 * it to %NULL.
 **/
void
ntd_free_and_nullify(gpointer *mem)
{
    if (*mem) {
        g_free(*mem);
        *mem = NULL;
    }
}

static guchar
ENCODE(guchar ch)
{
    return ch == 0 ? 96 : ch+32;
}

/**
 * ntd_uuencode:
 * @data: original binary buffer
 * @nbytes: size (in bytes) of the data to convert
 *
 * Converts @nbytes bytes from @data to their uu-encoded version.
 *
 * Returns: the uu-encoded '\0' terminated result of the conversion
 *          or %NULL on errors. Free the returned string with g_free()
 *          when no longer needed.
 **/
gchar *
ntd_uuencode(gconstpointer data, gsize nbytes)
{
    GString *encoded;
    const guchar *ptr;
    gssize n;
    guchar ch1, ch2, ch3;

    g_return_val_if_fail(data != NULL, NULL);

    encoded = g_string_sized_new(nbytes);
    g_string_assign(encoded, "");
    ptr = data;

    while (nbytes > 0) {
        n = MIN(45, nbytes);
        nbytes -= n;
        g_string_append_c(encoded, ENCODE(n));

        while (n > 0) {
            ch1 = *ptr++;             --n;
            ch2 = n > 0 ? *ptr++ : 0; --n;
            ch3 = n > 0 ? *ptr++ : 0; --n;
            g_string_append_c(encoded, ENCODE(ch1 >> 2));
            g_string_append_c(encoded, ENCODE(((ch1 & 0x3) << 4) + (ch2 >> 4)));
            g_string_append_c(encoded, ENCODE(((ch2 & 0xf) << 2) + (ch3 >> 6)));
            g_string_append_c(encoded, ENCODE(ch3 & 0x3f));
        }

        g_string_append_c(encoded, '\n');
    }

    return g_string_free(encoded, FALSE);
}

static gint
DECODE(guchar ch)
{
    if (ch > 96 || ch < 32)
        return -1;
    return ch == 96 ? 0 : ch - 32;
}

static const gchar *
ntd_uudecode_row(const gchar *row, GString *dst)
{
    gint n, ch1, ch2, ch3, ch4;

    n = DECODE(*row++);
    if (n < 0)
        return NULL;

    while (n > 0) {
        ch1 = DECODE(*row++);
        ch2 = DECODE(*row++);
        ch3 = DECODE(*row++);
        ch4 = DECODE(*row++);
        if (ch1 == -1 || ch2 == -1 || ch3 == -1 || ch4 == -1)
            return NULL;

        g_string_append_c(dst, (ch1 << 2) + (ch2 >> 4));
        --n;
        if (n > 0) {
            g_string_append_c(dst, ((ch2 & 0xf) << 4) + (ch3 >> 2));
            --n;
        }
        if (n > 0) {
            g_string_append_c(dst, ((ch3 & 0x3) << 6) + ch4);
            --n;
        }
    }

    return row;
}

/**
 * ntd_uudecode:
 * @encoded: '\0' terminated string of UU-encoded data
 * @p_data: pointer to the destination buffer to allocate
 * @p_nbytes: pointer to the destination size (in bytes) of the data
 *
 * Decodes the @encoded string and returns the decoded result
 * in @p_nbytes and @p_data. Either @p_nbytes and @p_data can
 * be %NULL, in which case the respective result will not be
 * returned.
 *
 * Returns: %TRUE on success, %FALSE on errors.
 **/
gboolean
ntd_uudecode(const gchar *encoded, gpointer *p_data, gsize *p_nbytes)
{
    GString *decoded;
    const gchar *ptr;

    g_return_val_if_fail(encoded != NULL, FALSE);

    decoded = g_string_sized_new(strlen(encoded) / 4 * 3);
    g_string_assign(decoded, "");
    ptr = encoded;

    while (*ptr) {
        ptr = ntd_uudecode_row(ptr, decoded);
        if (ptr == NULL)
            break;

        /* Skip paddings and newlines */
        while (*ptr == '`' || *ptr == '\n' || *ptr == '\r')
            ++ptr;
    }

    if (ptr == NULL) {
        g_string_free(decoded, TRUE);
        return FALSE;
    }

    if (p_nbytes)
        *p_nbytes = decoded->len;

    if (p_data) {
        *p_data = g_string_free(decoded, FALSE);
    } else {
        g_string_free(decoded, TRUE);
    }

    return TRUE;
}
