/* ntdisp - NtD In System Programming.
 *
 * Copyright (C) 2012  Nicola Fontana <ntd at entidi.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library. If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef __NTDISP_UTILS_H__
#define __NTDISP_UTILS_H__


G_BEGIN_DECLS

#define NTD_SYSTEM_ERROR        ntd_system_error_quark()


#ifdef __GNUC__

#define _NTD_PRETTY_FUNCTION_S  "%s"
#define _NTD_PRETTY_FUNCTION    __PRETTY_FUNCTION__

#else /* ! __GNUC__ */

#define _NTD_PRETTY_FUNCTION_S  "file %s, line %d"
#define _NTD_PRETTY_FUNCTION    __FILE__, __LINE__

#endif /* __GNUC__ */


#define ntd_log_pileup(err) \
    g_log(G_LOG_DOMAIN, G_LOG_LEVEL_WARNING, \
          _NTD_PRETTY_FUNCTION_S ": internal error yet set to `%d' (%s)", \
          _NTD_PRETTY_FUNCTION, (err)->code, (err)->message)

#define ntd_return_on_pileup(err) \
    G_STMT_START{ \
        if ((err) != NULL) { \
            ntd_log_pileup(err); \
            return; \
        } \
    }G_STMT_END

#define ntd_return_val_on_pileup(err,val) \
    G_STMT_START{ \
        if ((err) != NULL) { \
            ntd_log_pileup(err); \
            return (val); \
        } \
    }G_STMT_END

#define ntd_empty(string) \
    ((string) == NULL || (string)[0] == '\0')

typedef enum {
    /* NtdSystemError specific */
    NTD_SYSTEM_ERROR_NONE,
    NTD_SYSTEM_ERROR_FAILED,

    /* Derived from errno values on POSIX.1 (2001 edition) */
    NTD_SYSTEM_ERROR_2BIG,
    NTD_SYSTEM_ERROR_ACCES,
    NTD_SYSTEM_ERROR_ADDRINUSE,
    NTD_SYSTEM_ERROR_ADDRNOTAVAIL,
    NTD_SYSTEM_ERROR_AFNOSUPPORT,
    NTD_SYSTEM_ERROR_AGAIN,
    NTD_SYSTEM_ERROR_ALREADY,
    NTD_SYSTEM_ERROR_BADF,
    NTD_SYSTEM_ERROR_BADMSG,
    NTD_SYSTEM_ERROR_BUSY,
    NTD_SYSTEM_ERROR_CANCELED,
    NTD_SYSTEM_ERROR_CHILD,
    NTD_SYSTEM_ERROR_CONNABORTED,
    NTD_SYSTEM_ERROR_CONNREFUSED,
    NTD_SYSTEM_ERROR_CONNRESET,
    NTD_SYSTEM_ERROR_DEADLK,
    NTD_SYSTEM_ERROR_DESTADDRREQ,
    NTD_SYSTEM_ERROR_DOM,
    NTD_SYSTEM_ERROR_EXIST,
    NTD_SYSTEM_ERROR_FAULT,
    NTD_SYSTEM_ERROR_FBIG,
    NTD_SYSTEM_ERROR_HOSTUNREACH,
    NTD_SYSTEM_ERROR_IDRM,
    NTD_SYSTEM_ERROR_ILSEQ,
    NTD_SYSTEM_ERROR_INPROGRESS,
    NTD_SYSTEM_ERROR_INTR,
    NTD_SYSTEM_ERROR_INVAL,
    NTD_SYSTEM_ERROR_IO,
    NTD_SYSTEM_ERROR_ISCONN,
    NTD_SYSTEM_ERROR_ISDIR,
    NTD_SYSTEM_ERROR_LOOP,
    NTD_SYSTEM_ERROR_MFILE,
    NTD_SYSTEM_ERROR_MLINK,
    NTD_SYSTEM_ERROR_MSGSIZE,
    NTD_SYSTEM_ERROR_NAMETOOLONG,
    NTD_SYSTEM_ERROR_NETDOWN,
    NTD_SYSTEM_ERROR_NETRESET,
    NTD_SYSTEM_ERROR_NETUNREACH,
    NTD_SYSTEM_ERROR_NFILE,
    NTD_SYSTEM_ERROR_NOBUFS,
    NTD_SYSTEM_ERROR_NODATA,
    NTD_SYSTEM_ERROR_NODEV,
    NTD_SYSTEM_ERROR_NOENT,
    NTD_SYSTEM_ERROR_NOEXEC,
    NTD_SYSTEM_ERROR_NOLCK,
    NTD_SYSTEM_ERROR_NOMEM,
    NTD_SYSTEM_ERROR_NOMSG,
    NTD_SYSTEM_ERROR_NOPROTOOPT,
    NTD_SYSTEM_ERROR_NOSPC,
    NTD_SYSTEM_ERROR_NOSR,
    NTD_SYSTEM_ERROR_NOSTR,
    NTD_SYSTEM_ERROR_NOSYS,
    NTD_SYSTEM_ERROR_NOTCONN,
    NTD_SYSTEM_ERROR_NOTDIR,
    NTD_SYSTEM_ERROR_NOTEMPTY,
    NTD_SYSTEM_ERROR_NOTSOCK,
    NTD_SYSTEM_ERROR_NOTSUP,
    NTD_SYSTEM_ERROR_NOTTY,
    NTD_SYSTEM_ERROR_NXIO,
    NTD_SYSTEM_ERROR_OPNOTSUPP,
    NTD_SYSTEM_ERROR_OVERFLOW,
    NTD_SYSTEM_ERROR_PERM,
    NTD_SYSTEM_ERROR_PIPE,
    NTD_SYSTEM_ERROR_PROTO,
    NTD_SYSTEM_ERROR_PROTONOSUPPORT,
    NTD_SYSTEM_ERROR_PROTOTYPE,
    NTD_SYSTEM_ERROR_RANGE,
    NTD_SYSTEM_ERROR_ROFS,
    NTD_SYSTEM_ERROR_SPIPE,
    NTD_SYSTEM_ERROR_SRCH,
    NTD_SYSTEM_ERROR_TIME,
    NTD_SYSTEM_ERROR_TIMEDOUT,
    NTD_SYSTEM_ERROR_TXTBSY,
    NTD_SYSTEM_ERROR_XDEV
} NtdSystemError;


GQuark          ntd_system_error_quark          (void);
NtdSystemError  ntd_system_get_error            (void);
gchar*          ntd_system_get_error_message    (NtdSystemError error_code);
gboolean        ntd_system_set_error            (GError**       error,
                                                 const gchar*   format,
                                                 ...) G_GNUC_PRINTF (2, 3);
gboolean        ntd_system_set_errorv           (GError**       error,
                                                 const gchar*   format,
                                                 va_list        args);
void            ntd_system_set_error_with_code  (GError**       error,
                                                 NtdSystemError error_code,
                                                 const gchar*   format,
                                                 ...) G_GNUC_PRINTF (3, 4);
void            ntd_system_set_errorv_with_code (GError**       error,
                                                 NtdSystemError error_code,
                                                 const gchar*   format,
                                                 va_list        args);
void            ntd_system_clear_error          (void);
void            ntd_set_verbose_mode            (gboolean       new_status);
gchar *         ntd_escape                      (gconstpointer  buffer,
                                                 gsize          nbytes);
gulong          ntd_checksum                    (gconstpointer  data,
                                                 gsize          nbytes);
gchar *         ntd_find_file                   (const gchar*   file,
                                                 ...) G_GNUC_NULL_TERMINATED;
void            ntd_string_free                 (GString*       string,
                                                 gpointer*      p_buffer,
                                                 gsize*         p_len);
void            ntd_free_and_nullify            (gpointer*      mem);
gchar *         ntd_uuencode                    (gconstpointer  data,
                                                 gsize          nbytes);
gboolean        ntd_uudecode                    (const gchar*   encoded,
                                                 gpointer*      p_data,
                                                 gsize*         p_nbytes);

G_END_DECLS


#endif /* __NTDISP_UTILS_H__ */
