/* ntdisp - NtD In System Programming.
 *
 * Copyright (C) 2012  Nicola Fontana <ntd at entidi.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library. If not, see <http://www.gnu.org/licenses/>.
 */


#include <ntdisp.h>

static void
check_escape(void)
{
    gchar *result;

    result = ntd_escape("", 0);
    g_assert_cmpstr(result, ==, "");
    g_free(result);

    result = ntd_escape("Test1234", 8);
    g_assert_cmpstr(result, ==, "Test1234");
    g_free(result);

    result = ntd_escape("", 1);
    g_assert_cmpstr(result, ==, "\\000");
    g_free(result);

    result = ntd_escape("\0", 2);
    g_assert_cmpstr(result, ==, "\\000\\000");
    g_free(result);

    result = ntd_escape("\0\t\x0d\x0a", 5);
    g_assert_cmpstr(result, ==, "\\000\\t\\r\\n\\000");
    g_free(result);

    result = ntd_escape("abc\0def", 7);
    g_assert_cmpstr(result, ==, "abc\\000def");
    g_free(result);

    result = ntd_escape("\x01\x02\xFE\xFF", 4);
    g_assert_cmpstr(result, ==, "\\001\\002\\376\\377");
    g_free(result);
}

gint
main(gint argc, gchar *argv[])
{
    check_escape();
    return 0;
}
