/* ntdisp - NtD In System Programming.
 *
 * Copyright (C) 2012, 2013  Nicola Fontana <ntd at entidi.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library. If not, see <http://www.gnu.org/licenses/>.
 */


#include "../src/config.h"
#include <ntdisp.h>
#include <gio/gio.h>

#define G_SETTINGS_ENABLE_BACKEND
#include <gio/gsettingsbackend.h>

static void
check_defaults(GSettings *settings)
{
    GVariant *variant;

    variant = g_settings_get_value(settings, "device");
    g_assert(variant != NULL);
    g_assert_cmpstr(g_variant_get_string(variant, NULL), ==, "");

    variant = g_settings_get_value(settings, "protocol");
    g_assert(variant != NULL);
    g_assert_cmpstr(g_variant_get_string(variant, NULL), ==, "NtdIsp");
    g_variant_unref(variant);

    variant = g_settings_get_value(settings, "init");
    g_assert(variant != NULL);
    g_assert(g_variant_get_boolean(variant));
    g_variant_unref(variant);

    variant = g_settings_get_value(settings, "action");
    g_assert(variant != NULL);
    g_assert_cmpstr(g_variant_get_string(variant, NULL), ==, "Null");
    g_variant_unref(variant);


    variant = g_settings_get_value(settings, "save-to");
    g_assert(variant != NULL);
    g_assert_cmpstr(g_variant_get_string(variant, NULL), ==, "");
    g_variant_unref(variant);

    variant = g_settings_get_value(settings, "read-from");
    g_assert(variant != NULL);
    g_assert(! g_variant_get_boolean(variant));
    g_variant_unref(variant);

    variant = g_settings_get_value(settings, "read-from-address");
    g_assert(variant != NULL);
    g_assert_cmpstr(g_variant_get_string(variant, NULL), ==, "");
    g_variant_unref(variant);

    variant = g_settings_get_value(settings, "read-size");
    g_assert(variant != NULL);
    g_assert(! g_variant_get_boolean(variant));
    g_variant_unref(variant);

    variant = g_settings_get_value(settings, "read-size-bytes");
    g_assert(variant != NULL);
    g_assert_cmpstr(g_variant_get_string(variant, NULL), ==, "");
    g_variant_unref(variant);

    variant = g_settings_get_value(settings, "load-from");
    g_assert(variant != NULL);
    g_assert_cmpstr(g_variant_get_string(variant, NULL), ==, "");
    g_variant_unref(variant);

    variant = g_settings_get_value(settings, "erase");
    g_assert(variant != NULL);
    g_assert(g_variant_get_boolean(variant));
    g_variant_unref(variant);

    variant = g_settings_get_value(settings, "write-to");
    g_assert(variant != NULL);
    g_assert(! g_variant_get_boolean(variant));
    g_variant_unref(variant);

    variant = g_settings_get_value(settings, "write-to-address");
    g_assert(variant != NULL);
    g_assert_cmpstr(g_variant_get_string(variant, NULL), ==, "");
    g_variant_unref(variant);

    variant = g_settings_get_value(settings, "verify");
    g_assert(variant != NULL);
    g_assert(! g_variant_get_boolean(variant));
    g_variant_unref(variant);

    variant = g_settings_get_value(settings, "execute");
    g_assert(variant != NULL);
    g_assert(! g_variant_get_boolean(variant));
    g_variant_unref(variant);
}

static void
check_sets(GSettings *settings)
{
    /* Check some protocol values */
    g_assert(g_settings_set(settings, "protocol", "s", "NtdLpc"));
    g_assert_cmpstr(g_settings_get_string(settings, "protocol"), ==, "NtdLpc");
    g_assert(g_settings_set(settings, "protocol", "s", "NtdStm"));
    g_assert_cmpstr(g_settings_get_string(settings, "protocol"), ==, "NtdStm");
    g_assert(! g_settings_set(settings, "protocol", "s", "unexistent class"));
    g_assert_cmpstr(g_settings_get_string(settings, "protocol"), ==, "NtdStm");

    /* Check some action values */
    g_assert(g_settings_set(settings, "action", "s", "Read"));
    g_assert_cmpstr(g_settings_get_string(settings, "action"), ==, "Read");
    g_assert(g_settings_set(settings, "action", "s", "Write"));
    g_assert_cmpstr(g_settings_get_string(settings, "action"), ==, "Write");
    g_assert(! g_settings_set(settings, "action", "s", "unexistent action"));
    g_assert_cmpstr(g_settings_get_string(settings, "action"), ==, "Write");
}

gint
main(gint argc, gchar *argv[])
{
    GSettingsBackend *backend;
    GSettings        *settings;

    g_type_init();

    /* Added to silent a spurious warning in loading memory backend */
    g_io_extension_point_register(G_SETTINGS_BACKEND_EXTENSION_POINT_NAME);

    /* Test memory backend */
    backend = g_memory_settings_backend_new();
    g_assert(backend != NULL);

    settings = g_settings_new_with_backend(NTDISP_SCHEMA, backend);
    g_object_unref(backend);
    g_assert(settings != NULL);

    check_defaults(settings);
    check_sets(settings);
    g_object_unref(settings);

    return 0;
}
