/* ntdisp - NtD In System Programming.
 *
 * Copyright (C) 2012  Nicola Fontana <ntd at entidi.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library. If not, see <http://www.gnu.org/licenses/>.
 */


#include <ntdisp.h>

static void
check_encoding(const gchar *file, const gchar *uu_file)
{
    gchar *buffer, *uu_buffer, *my_buffer;
    gsize size, uu_size;

    g_assert(g_file_get_contents(file, & buffer, & size, NULL));
    g_assert(g_file_get_contents(uu_file, & uu_buffer, & uu_size, NULL));

    my_buffer = ntd_uuencode(buffer, size);
    g_free(buffer);

    g_assert_cmpstr(my_buffer, ==, uu_buffer);
    g_free(uu_buffer);
    g_free(my_buffer);
}

static void
check_decoding(const gchar *uu_file, const gchar *file)
{
    gchar *buffer, *uu_buffer;
    gpointer my_buffer;
    gsize size, uu_size, my_size;

    g_assert(g_file_get_contents(file, & buffer, & size, NULL));
    g_assert(g_file_get_contents(uu_file, & uu_buffer, & uu_size, NULL));

    /* Check it works without passing any destination */
    g_assert(ntd_uudecode(uu_buffer, NULL, NULL));

    g_assert(ntd_uudecode(uu_buffer, & my_buffer, & my_size));
    g_free(uu_buffer);

    if (my_buffer == NULL)
        my_buffer = g_strdup("");

    g_assert_cmpuint(my_size, ==, size);

    g_assert_cmpstr(my_buffer, ==, buffer);
    g_free(buffer);
    g_free(my_buffer);
}

gint
main(gint argc, gchar *argv[])
{
    check_encoding(SRCDIR "/test0",    SRCDIR "/test0.uu");
    check_encoding(SRCDIR "/test1",    SRCDIR "/test1.uu");
    check_encoding(SRCDIR "/test2",    SRCDIR "/test2.uu");
    check_encoding(SRCDIR "/test3",    SRCDIR "/test3.uu");

    check_decoding(SRCDIR "/test0.uu", SRCDIR "/test0");
    check_decoding(SRCDIR "/test1.uu", SRCDIR "/test1");
    check_decoding(SRCDIR "/test2.uu", SRCDIR "/test2");
    check_decoding(SRCDIR "/test3.uu", SRCDIR "/test3");

    return 0;
}
